#!/usr/bin/python
# -*- coding: utf-8 -*-
'''
Created on Fri May 13 13:06:08 2016

@author: Jeremy Yon & Frédéric Hérault
CONTACT: frederic.herault@inrae.fr
'''



import subprocess
from scipy.cluster.hierarchy import linkage, to_tree
from scipy.spatial.distance import pdist, squareform
import argparse
import time
import os


###########  Parser for command-line options, arguments and sub-commands ###########

parser = argparse.ArgumentParser(
formatter_class=argparse.RawDescriptionHelpFormatter,
description="Description: Select SNP for Imputation",
epilog='''\

-------------------
Input Genotype file PLINK format=.ped
The PED file is a white-space (space or tab) delimited file: the first six columns are mandatory:
		Family ID
		Individual ID
		Paternal ID
		Maternal ID
		Sex (1=male; 2=female; other=unknown)
		Phenotype
		Genotype
Genotypes (column 7 onwards) should also be white-space delimited; they can be any character (e.g. 1,2,3,4 or A,C,G,T or anything else) except 0 which is, by default, the missing genotype character. All markers should be biallelic. All SNPs (whether haploid or not) must have two alleles specified. Either Both alleles should be missing (i.e. 0) or neither. No header row should be given.

ex:

Uto_124 Uto_124 0 0 0 -9 G G T T T T
Uto_13 Uto_13 0 0 0 -9 G G T T C T
Uto_243 Uto_243 0 0 0 -9 A G T T T T
Uto_244 Uto_244 0 0 0 -9 G G T T C T
Uto_273 Uto_273 0 0 0 -9 G G T T T T

By default, each line of the MAP file describes a single marker and must contain exactly 4 columns:

		chromosome
		snp identifier
		Genetic distance (morgans)
		Base-pair position (bp units)

ex:

25	AX-77283174	0	5208
24	AX-80751587	0	6369
28	AX-76388168	0	6695
28	AX-76388358	0	7122
25	AX-80909995	0	7629


''')

# argument positionnel obligatoire
parser.add_argument("Parameter", help="Path to Parameter file")
args = parser.parse_args()


########### FUNCTIONS ###########

### Functions Read PARAMETERS FILE

def Read_Parameters_File(Parameters_File_Path):
	'''Fonction qui prend en charge la lecture du fichier parameters.txt
	et renvoie les parametres de l'analyse.'''
	Input_File =''
	Var_Path_Geno = ''
	Var_Path_Map = ''
	Var_Threshold = ''
	Var_Chr = ''
	Var_Path_Out = ''
	Var_Seuil_Nb_SNP=''
	List_Chr=[]
	Input_File = open(Parameters_File_Path, "r")
	for ligne in Input_File:
		ligne = ligne.split()
		if(ligne == []):
			continue
		if(ligne[0] == 'in_geno'):
			Var_Path_Geno = ligne[2]
		if(ligne[0] == 'in_map'):
			Var_Path_Map = ligne[2]
		if(ligne[0] == 'threshold'):
			Var_Threshold = float(ligne[2])
		if(ligne[0] == 'chromosome'):
			Var_Chr = ligne[2]
			List_Chr = Create_List_Chromosome_to_Analyze(Var_Chr, Var_Path_Map)
		if(ligne[0] == 'path_out'):
			Var_Path_Out = ligne[2]
		if (ligne[0] == 'SNP_window'):
			Var_Seuil_Nb_SNP= int(ligne[2])
	Input_File.close()
	return Var_Path_Geno, Var_Path_Map, Var_Threshold, List_Chr, Var_Chr, Var_Path_Out, Var_Seuil_Nb_SNP

def Create_List_Chromosome_to_Analyze(Parameters_Chromosome_List, Map_File_Path):
	''' A partir de l'argument chromosome du fichier parametre et du fichier map
	renvoie une liste des chromosomes à analyser.
	Supprime le chr 0 si présent dans le fichier map
	pour travailler spécifiquement sur le chr 0 il ne faut pas spécifier all dans le fichier parametre'''
	Input_File =''
	List_chr = []
	if (Parameters_Chromosome_List == 'all'):
		Input_File = open(Map_File_Path, "r")
		for ligne in Input_File:
			ligne = ligne.split()
			ligne_0=ligne[0]
			if (ligne_0.isdigit()):
				ligne_0=int(ligne_0)
			if(ligne_0!= 0 and ligne_0 not in List_chr):
				List_chr.append(ligne_0)
		Input_File.close()
	else:
		List=Parameters_Chromosome_List.split(",")
		for elt in List:
			if (elt.isdigit()):
				List_chr.append(int(elt))
			else:
				List_chr.append(elt)
	List_chr = sorted(List_chr)
	List_chr=map(str,List_chr)
	return List_chr



### Fonctions extraction SNP et position sur un chromosome

def Get_List_Tuple_Position_SNPName(Chrom_Name, Map_File_Path):
	'''Fonction qui crée une liste de tuple contenant le nom et la position des SNP présent sur le chromosome "Chrom_Name" en se basant sur la carte fournie dans le fichier de parametre'''
	List_Tuple_Position_SNPName=[]
	Input_File = open(Map_File_Path, "r")
	for ligne in Input_File:
		ligne = ligne.split()
		if(ligne[0] == Chrom_Name):
			List_Tuple_Position_SNPName.append((float(ligne[3]),ligne[1]))
	List_Tuple_Position_SNPName=sorted(List_Tuple_Position_SNPName)
	return List_Tuple_Position_SNPName



### Fonctions préparation liste SNP pour PLINK

def Extract_SNP_List_for_PLINK(List_Tuple_Position_SNPName, Nb_SNP_to_Analyze, Output_Path):
	'''Fonction qui crée un fichier contenant la liste des N SNP à analyser'''
	Output_snp_name = Output_Path+"snp_name.txt"
	Output_snp_name = open (Output_snp_name, "w")
	for Tuple in List_Tuple_Position_SNPName [: Nb_SNP_to_Analyze]:
		Output_snp_name.write(Tuple[1]+'\n')
	Output_snp_name.close()



### Fonctions IMPORTATION DES DONNEES R2 et freq issus de PLINK et creation fichier position des SNP

def Import_R2_Matrix(R2_Matrix_Path):
	''' Importation des données R2 généré par PLINK'''
	List_data_R2 = []
	Nb_SNP=0 #correspond à l'index du snp
	Input_File = open(R2_Matrix_Path, "r")
	for ligne in Input_File:
		ligne_float = [] #recupère la ligne en valeur float
		ligne_decoupe = ligne.split() #sépare les elt
		for elt in ligne_decoupe[Nb_SNP+1:]: #ajout des valeurs du snp + passage de str en float
			ligne_float.append(float(elt))
		List_data_R2.append(ligne_float)
		Nb_SNP += 1
	Input_File.close()
	return List_data_R2, Nb_SNP

def Import_MAF(Freq_File_Path):
	''' Importation des MAF calculée par PLINK'''
	List_SNP_Name = []
	List_Maf = []
	Input_File = open(Freq_File_Path, "r")
	count = 0
	for line in Input_File:
		if(count != 0): #permet de ne pas selectionner la premiere ligne du fichier
			line_split = line.split()
			List_SNP_Name.append(line_split[1])
			List_Maf.append(line_split[4])
		count += 1
	Input_File.close()
	return List_SNP_Name, List_Maf

def Extract_SNP_Position(Chrom_Name, Map_File_Path):
	'''Fonction qui crée un fichier contenant les positions ordonnées des SNP sur le chromosome "Chrom_Name" sur la carte fournie dans le fichier de parametre.
	L'ordre des position est croissant et correspond à l'ordre des SNP fourni par PLINK dans les fichier MAF ou r2 matrix'''
	List_Position = []
	Input_File = open(Map_File_Path, "r")
	for ligne in Input_File:
		ligne = ligne.split()
		if(ligne[0] == Chrom_Name):
			List_Position.append(int (float(ligne[3])))
	List_Position.sort()
	Input_File.close()
	return List_Position



### Fonctions Nettoyage des données

def Get_Index_of_SNP_to_Delete(List_data_R2, Nb_SNP, R2_Threshold):
	'''Récupère les noms de snp que l'on doit enlever de l'analyse.'''
	List_Index_of_SNP_to_Delete = [] # récupère les snp tout le temps sous le seuil
	for Index in range(Nb_SNP): #pour tester tous les snp à partir de leur indice
		SNP_R2_Values = Get_SNP_R2_Values(List_data_R2, Index) #reforme une liste de toutes les valeurs du SNP
		if(Test_SNP_vs_Threshold(SNP_R2_Values, R2_Threshold)): #si aucune valeurs de R2 du snp ne dépasse le seuil
			List_Index_of_SNP_to_Delete.insert(0, Index)  #ajout de son indice dans la liste, ordre décroissant
	return List_Index_of_SNP_to_Delete

def Get_SNP_R2_Values(List_data_R2, SNP_Index):
	'''récupère toutes les valeurs d'un snp.'''
	SNP_R2_Values = []
	x = 1 # récupérer la valeur sur la bonne colonne
	for i in range(SNP_Index): #i selectionne la ligne
		SNP_R2_Values.append(List_data_R2[i][SNP_Index-x]) #ajout valeur colonne
		x +=1
	SNP_R2_Values.extend(List_data_R2[SNP_Index]) #ajout valeurs ligne
	return SNP_R2_Values

def Test_SNP_vs_Threshold(List_R2_Values, R2_Threshold):
	'''snp_vs_seuil renvoit un boolean True si aucune valeur du snp n'est supérieure au seuil.'''
	Answer = True #initialisation de la réponse
	for R2_Values in List_R2_Values:
		if(R2_Values >= R2_Threshold): #if max > seuil
			Answer = False
			break
	return Answer

def Cleaning_Data(List_data_R2, List_SNP_Name, List_MAF, List_Position,List_Index_of_SNP_to_Delete):
	'''Supprime de l'analyse les snp qui ne dépasse jamais le seuil de DL fixé.'''
	List_Name_SNP_Deleted = []
	for Index in List_Index_of_SNP_to_Delete:
		List_Name_SNP_Deleted.append(List_SNP_Name[Index])
		del List_data_R2[Index]
		del List_SNP_Name[Index]
		del List_MAF[Index]
		del List_Position[Index]
		x = 0 #indice des colonnes à supprimer
		Index = Index-1 #commence à supprimer sur la ligne du dessous du snp
		while(Index >= 0):
			del List_data_R2[Index][x]
			Index = Index - 1
			x = x + 1
	return List_data_R2, List_SNP_Name, List_MAF, List_Position, List_Name_SNP_Deleted




### Fonctions  CONSTRUCTION DU DENDROGRAM

def List_data_R2_to_Flat_Distance_Array(List_data_R2):
	'''Transforme les listes de données de chaque snp en une unique liste de distance (1-R²).
	Ce format est necessaire pour utiliser la fonction linkage de Scipy.'''
	Flat_Distance_Array = []
	for List_R2_Values in List_data_R2:
		for R2_Values in List_R2_Values:
			Flat_Distance_Array.append(1-R2_Values)
	return Flat_Distance_Array



### Fonctions PARCOURS DU DENDROGRAM A LA RECHERCHE DES CLUSTERS

def Cluster_Selection(Z, List_data_R2, R2_Threshold):
	"""Cette fonction permet de parcourir le dendrogramme en partant de la racine. Le parcours utilise la méthode Glouton afin de parcourir l'ensemble des noeuds.
	A chaque noeud la fonction examine si ce noeud correspond à un ensemble de feuilles à selectionner."""
	Main_Tree = to_tree(Z)
	Parcours = ['g']
	while( 1 == 1):
		# Création sous arbres et test les clusters
		Sub_Tree = Get_Sub_Tree(Main_Tree, Parcours) #on crée un sous arbre
		Cluster = Identify_Leaves_of_Sub_Tree(Main_Tree, Sub_Tree, Z) #récupère les feuilles composants le noeud sélectionné
		R2_Matrix_Sub_Tree = Get_R2_Matrix_of_Sub_Tree(Main_Tree, Sub_Tree, Z, List_data_R2) # création de la sous matrice pour le noeud testé
		if(Cluster_Validation(R2_Matrix_Sub_Tree, R2_Threshold) == True): #test si toutes les valeurs de la sous matrice sont supérieures au seuil
			if(len(Cluster) == 1):# si singleton
				Selected_SNP_Index=Cluster[0]
				Selected_SNP_Name=List_SNP_Name[Selected_SNP_Index]
				Selected_SNP_Position=List_Position[Selected_SNP_Index]
				Selected_SNP_MAF=List_MAF[Selected_SNP_Index]
				Nb_SNP_in_cluster=1
				# ecrit les resultats
				Output_Result_File=open (Var_Path_Result_out, "a")
				Output_Result_File.write(Selected_SNP_Name+"\t"+Var_Chrom_Name+"\t"+str(Selected_SNP_Position)+"\t"+str(Selected_SNP_MAF)+"\t"+str(Nb_SNP_in_cluster)+"\n")
				Output_Result_File.close()
			else:
				Selected_SNP_Index = SNP_Selection(Cluster, List_MAF, List_Position)
				Selected_SNP_Name=List_SNP_Name[Selected_SNP_Index]
				Selected_SNP_Position, Selected_SNP_MAF, Selected_SNP_Min_R2, Selected_SNP_Max_R2, Selected_SNP_Mean_R2, Selected_SNP_SD_R2, Selected_SNP_Distance_Min, Selected_SNP_Distance_Max, Selected_SNP_Distance_Mean, Selected_SNP_Distance_SD=Result_for_Selected_SNP(Selected_SNP_Index, Cluster, List_data_R2, List_Position, List_MAF)
				R2_min_in_cluster, R2_mean_in_cluster, R2_sd_in_cluster, Nb_SNP_in_cluster, Distance_cluster_min, Distance_cluster_max, Distance_cluster_mean, Distance_cluster_sd=Result_for_Cluster(Cluster, R2_Matrix_Sub_Tree )
				# ecrit les resultats
				Output_Result_File=open (Var_Path_Result_out, "a")
				Output_Result_File.write(Selected_SNP_Name+"\t"+Var_Chrom_Name+"\t"+str(Selected_SNP_Position)+"\t"+str(Selected_SNP_MAF)+"\t"+str(Nb_SNP_in_cluster)+"\t"+str(R2_min_in_cluster)+"\t"+str(R2_mean_in_cluster)+"\t"+str(R2_sd_in_cluster)+"\t"+str(Selected_SNP_Min_R2)+"\t"+str(Selected_SNP_Max_R2)+"\t"+str(Selected_SNP_Mean_R2)+"\t"+str(Selected_SNP_SD_R2)+"\t"+str(Distance_cluster_min)+"\t"+str(Distance_cluster_max)+"\t"+str(Distance_cluster_mean)+"\t"+str(Distance_cluster_sd)+"\t"+str(Selected_SNP_Distance_Min)+"\t"+str(Selected_SNP_Distance_Max)+"\t"+str(Selected_SNP_Distance_Mean)+"\t"+str(Selected_SNP_Distance_SD)+"\n")
				Output_Result_File.close()
# reprends le parcours
			while(Parcours[-1] == 'd'): #methode glouton : lorsque la derniere lettre du parcours est un d et que cluster de feuille a été retenu, on supprime le d.
				del Parcours[len(Parcours)-1]
				if(len(Parcours) == 0): #si parcours ne contient aucune lettre, on fait un break pour arreter la boucle infinie
					break
			if(len(Parcours) == 0): #idem, break la boucle infinie
				break
			Parcours[-1] = 'd' #remplace la derniere lettre par un d parceque le cluster a été selectionné et qu'il faut aller au dernier noeud à droite non analysé.
		else:
			Parcours.append('g') #le cluster n'a pas été retenu d'où l'ajout d'un g pour continuer à explorer l'arbre par la méthode Glouton

def Get_Sub_Tree(Main_Tree, Parcours): #permet de créer un sous arbre à partir du dendrogramme
	"""Cette fonction permet de créer un arbre en partant de la racine (main_tree) à l'aide d'un parcours.
	Dans le parcours un g signifie que l'arbre suivant (descendant) est l'arbre fils de gauche, d signifit l'arbre de droite"""
	Sub_Tree = Main_Tree
	for Elt_Parcours in Parcours:
		if(Elt_Parcours == 'g'):
			Sub_Tree = Sub_Tree.get_left()
		if(Elt_Parcours == 'd'):
			Sub_Tree = Sub_Tree.get_right()
	return Sub_Tree



### Fonctions CREATION DES SOUS MATRICES LD PAR NOEUD

def Get_R2_Matrix_of_Sub_Tree(Main_Tree, Sub_Tree, Z, List_data_R2): #id_node = Z = tree
	"""Cette fonction crée une matrice avec les feuilles d'un noeud.
	Elle renvoit une matrice à 2 dimension avec n observation, n est égale au nombre de feuilles."""
	List_Leaves_of_Sub_Tree = Identify_Leaves_of_Sub_Tree(Main_Tree, Sub_Tree, Z)
	Sub_Matrix = [] #réponse
	for Leaf in List_Leaves_of_Sub_Tree:
		for i in List_Leaves_of_Sub_Tree:
			if(i == Leaf): #ne met pas sa valeur contre lui meme (1)
				continue
			if(Leaf > i):
				continue
			if(Leaf < i):
				Sub_Matrix.append(List_data_R2[Leaf][i-Leaf-1])
	return Sub_Matrix

def Cluster_Validation(R2_Matrix_Sub_Tree, R2_Threshold):
	"""Cette fonction renvoit Vrai si toutes les valeurs de la matrice sont au dessus du seuil."""
	Validate = True
	for R2_Value in R2_Matrix_Sub_Tree:
		if(R2_Value < R2_Threshold):
			Validate = False
			break
	return Validate

### Fonctions METHODE POUR ACCEDER AUX feuilles (index SNP)DU DENDROGRAM

def Identify_Leaves_of_Sub_Tree(Main_Tree, Sub_Tree, Z):
	"""Cette fonction identifie les feuilles qui forment un noeud. Au minimum, un noeud est formé de deux feuilles.
	Cette fonction utilise les méthodes get_id() et get_count() provenant de scipy.cluster.hierarchy.ClusterNode.
	get_id() nous donne l'identifiant du noeud. (identifiant provenant de Z)
	get_count() nous indique le nombre de feuilles (d'observation) composant l'arbre.
	Cette fonction retourne une liste correspondant aux id des feuilles. (id = indice dans List_SNP_Name)
	"""
	List_Leaves_of_Sub_Tree = []
	if(Sub_Tree.get_count() == 1): #si le noeud a une seule feuille donc c'est une feuille
		# List_Leaves_of_Sub_Tree = []
		List_Leaves_of_Sub_Tree.append(Sub_Tree.get_id()) #emplacement du noeud avec l'indice cluster qui représente le n° feuille
	else:
		# List_Leaves_of_Sub_Tree = []
		Node = Sub_Tree.get_id()
		List_Leaves_of_Sub_Tree.append(int(Z[Node-Main_Tree.get_count(),0])) #récupère l'id de la première feuille de l'assemblage
		List_Leaves_of_Sub_Tree.append(int(Z[Node-Main_Tree.get_count(),1]))
		check = Control_Leaves(Sub_Tree, List_Leaves_of_Sub_Tree)
		while(check == False):
			List_Leaves_of_Sub_Tree = Identify_New_Leaves(Main_Tree, Z, List_Leaves_of_Sub_Tree)
			check = Control_Leaves(Main_Tree, List_Leaves_of_Sub_Tree)
	return List_Leaves_of_Sub_Tree

def Control_Leaves(Tree, List_Leaves):
	"""Renvoit vrai si toutes les feuilles d'un noeud sont identifiées."""
	Reply = True
	for Elt in List_Leaves:
		if(Elt > Tree.get_count()-1): #permet de savoir si l'elt est une feuille ou un assemblage de feuille (voir doc linkage)
			Reply = False
	return Reply

def Identify_New_Leaves(Main_Tree, Z, List_Leaves_in):
	"""La fonction récupère l'id des feuilles provenant d'un assemblage de feuilles.
	Elle renvoit une liste contenant le numéro des feuilles."""
	List_Leaves_out = [] #stock les feuilles
	for Elt in List_Leaves_in:
		Elt = int(Elt)  #cast elt en int afin de ne pas avoir d'erreur pour Z[elt-main_tree.get_count()]
		if(Elt > Main_Tree.get_count()-1): #permet de savoir si l'elt est une feuille ou un assemblage de feuille (voir doc linkage)
			List_Leaves_out.append(int(Z[Elt-Main_Tree.get_count(),0])) #récupère l'id de la première feuille de l'assemblage
			List_Leaves_out.append(int(Z[Elt-Main_Tree.get_count(),1])) #récupère la seconde
		else:
			List_Leaves_out.append(Elt)
	return List_Leaves_out




### Fonctions SELECTION D'UN SNP PAR CLUSTER

def SNP_Selection(Cluster, List_MAF, List_Position):
	'''Cette fonction permet de selectionner un snp par cluster. Cette selection se fait sur leur score de maf et leur distance entre eux.
	On selectionne le snp qui a la plus forte maf et qui proche de tous les autres.'''
	Selected_SNP_Index = ''
	List_MAF_Cluster = [] #contient les valeurs de maf par cluster
	List_Pos_Cluster = [] #contient valeurs de position par cluster
	Var_Max_MAF = 0 #initialisation de la valeur max de la maf pour un cluster
	Var_Index = 0 #récupère l'indice de la Var_max_maf
	Var_Distance_Mean = 0
	Var_Distance_SD = 0
	for SNP_Index in Cluster: #parcours des élémenents (id_snp) de la liste
		List_MAF_Cluster.append(List_MAF[SNP_Index]) #ajout des maf dans la liste
		List_Pos_Cluster.append([List_Position[SNP_Index]]) #comme pdist a besoin 2-dimension tab, on met la position dans une liste,même si elle est seule
	List_Distance_Cluster = pdist(List_Pos_Cluster) #liste de toutes les distances 2à2
	Var_Distance_Mean = Mean_Calculation(List_Distance_Cluster) #la moyenne des distances 2à2 entre tous les SNP du cluster
	Var_Distance_SD = SD_calculation(List_Distance_Cluster)
	Matrix_Distance_Cluster = squareform(List_Distance_Cluster)
	for i, elt in enumerate(List_MAF_Cluster):
		if(elt > Var_Max_MAF):
			Var_Distance_Mean_SNP = sum(Matrix_Distance_Cluster[i,])/(len(Matrix_Distance_Cluster[0,])-1) #la moyenne des distances par rapport aux autres SNP du cluster
			if(Var_Distance_Mean - Var_Distance_SD*2 < Var_Distance_Mean_SNP < Var_Distance_Mean + Var_Distance_SD*2):
				Var_Max_MAF = elt
				Var_Index = i
	Selected_SNP_Index=Cluster[Var_Index]
	return Selected_SNP_Index



### Fonctions FONCTIONS MATHEMATIQUES

def Mean_Calculation(List_Values):
	return sum(List_Values) / len(List_Values)

def Variance_Calculation(List_Values):
	m=Mean_Calculation(List_Values)
	return Mean_Calculation([(x-m)**2 for x in List_Values])

def SD_calculation(List_Values):
	return Variance_Calculation(List_Values)**0.5



### Fonctions FONCTION POUR LES RESULTATS

def Result_for_Selected_SNP(Selected_SNP_Index, Cluster, List_data_R2, List_Position, List_MAF):
	''' Cette fonction récupère des informations pour les résultats. Elle donne la position du SNP selectionné,
	sa maf, son R2 minimum et maximum avec les autres elements du cluster, la moyenne de son R2 et son écart type.
	Elle nous informe aussi sur la distance minimum et maximum entre le snp sélectionné et les autres snp présent dans le cluster.
	Ainsi que la moyenne et l'écart-Type de la distance intra Cluster.'''
	Selected_SNP_Name=List_SNP_Name[Selected_SNP_Index]
	Selected_SNP_Position=List_Position[Selected_SNP_Index]
	Selected_SNP_MAF=List_MAF[Selected_SNP_Index]
	Selected_SNP_Min_R2 = ''
	Selected_SNP_Max_R2 = ''
	Selected_SNP_Mean_R2 = ''
	Selected_SNP_SD_R2 = ''
	Selected_SNP_Distance_Min = ''
	Selected_SNP_Distance_Max = ''
	Selected_SNP_Distance_Mean = ''
	Selected_SNP_Distance_SD = ''
	Selected_SNP_R2_Values_in_Cluster = [] #initialise la liste des valeurs r2 entre le snp selectionné et les autres du cluster
	Selected_SNP_Distance_in_Cluster = [] #liste pour récupérer les distances du snp select par rapport aux autres du cluster]
	for SNP_Index in Cluster: #pour chaque snp du cluster
			if(SNP_Index == Selected_SNP_Index):
				continue
			Selected_SNP_Distance_in_Cluster.append(abs(Selected_SNP_Position-List_Position[SNP_Index])) #abs() nous renvoit la valeur absolue
			if(SNP_Index > Selected_SNP_Index):
				Selected_SNP_R2_Values_in_Cluster.append(List_data_R2[Selected_SNP_Index][SNP_Index-Selected_SNP_Index-1]) #☺permet de récupérer juste la valeur r2 entre les 2 snp
			if(SNP_Index < Selected_SNP_Index):
				Selected_SNP_R2_Values_in_Cluster.append(List_data_R2[SNP_Index][Selected_SNP_Index-SNP_Index-1])
			Selected_SNP_Min_R2=(min(Selected_SNP_R2_Values_in_Cluster))
			Selected_SNP_Max_R2=(max(Selected_SNP_R2_Values_in_Cluster))
			Selected_SNP_Mean_R2=(Mean_Calculation(Selected_SNP_R2_Values_in_Cluster))
			Selected_SNP_SD_R2=(SD_calculation(Selected_SNP_R2_Values_in_Cluster))
			Selected_SNP_Distance_Min=(min(Selected_SNP_Distance_in_Cluster))
			Selected_SNP_Distance_Max=(max(Selected_SNP_Distance_in_Cluster))
			Selected_SNP_Distance_Mean=(Mean_Calculation(Selected_SNP_Distance_in_Cluster))
			Selected_SNP_Distance_SD=(SD_calculation(Selected_SNP_Distance_in_Cluster))
	return Selected_SNP_Position, Selected_SNP_MAF, Selected_SNP_Min_R2, Selected_SNP_Max_R2, Selected_SNP_Mean_R2, Selected_SNP_SD_R2, Selected_SNP_Distance_Min, Selected_SNP_Distance_Max, Selected_SNP_Distance_Mean, Selected_SNP_Distance_SD


def Result_for_Cluster(Cluster, R2_Matrix_of_Cluster ):
	'''Cette fonction est utilisé pour récupérer les résultats. Elle donne le R2 moyen au sein d'un cluster ainsi que son écart type et le nombre de snp dans le cluster.
	Nous informe aussi sur la distance mimimum et maximum qui existe entre les éléments du cluster, ainsi que la moyenne de ces distances et son écart type. '''
	R2_min_in_cluster = min(R2_Matrix_of_Cluster)
	R2_mean_in_cluster = Mean_Calculation(R2_Matrix_of_Cluster)
	R2_sd_in_cluster = SD_calculation(R2_Matrix_of_Cluster)
	Nb_snp_in_cluster = len(Cluster)
	Distance_cluster_min = ''
	Distance_cluster_max = ''
	Distance_cluster_mean = ''
	Distance_cluster_sd = ''
	pos_cluster_temp = [] #les positions des snp d'un cluster
	for elt in Cluster:
		pos_cluster_temp.append([List_Position[elt]])
	list_dist_cluster = pdist(pos_cluster_temp)
	Distance_cluster_min=(min(list_dist_cluster))
	Distance_cluster_max=(max(list_dist_cluster))
	Distance_cluster_mean=(Mean_Calculation(list_dist_cluster))
	Distance_cluster_sd=(SD_calculation(list_dist_cluster))
	return R2_min_in_cluster, R2_mean_in_cluster, R2_sd_in_cluster, Nb_snp_in_cluster, Distance_cluster_min, Distance_cluster_max, Distance_cluster_mean, Distance_cluster_sd



### Fonctions FONCTION affichage heure

def Print_time(time_local):
	hour=''
	minute=''
	day=''
	month=''
	if (len(list(str(time_local[3])))==1):
		hour='0'+str(time_local[3])
	else:
		hour=str(time_local[3])
	if (len(list(str(time_local[4])))==1):
		minute='0'+str(time_local[4])
	else:
		minute=str(time_local[4])
	if (len(list(str(time_local[2])))==1):
		day='0'+str(time_local[2])
	else:
		day=str(time_local[2])
	if (len(list(str(time_local[1])))==1):
		month='0'+str(time_local[1])
	else:
		month=str(time_local[1])
	year=str(time_local[0])
	return hour + "h"+ minute + " on "+month+"/"+day+"/"+year



########### MAIN ###########



##### Lecture du fichier parametre et Creation des variables Path

Var_Path_Parametre = args.Parameter
Var_Path_Genotype, Var_Path_Map, Var_Threshold, List_Chromosome_to_Analyze, Var_Chr, Var_Path_Output, Var_Seuil_Nb_SNP = Read_Parameters_File(Var_Path_Parametre)

#Controle de l'existence du repertoire de resultats et le cree si besoin
if not os.path.exists(Var_Path_Output):
	os.makedirs(Var_Path_Output)

# Création de variables Path
Var_Path_R2_File = Var_Path_Output+"sortie_plink_r2.ld"
Var_Path_Freq_File = Var_Path_Output+"sortie_plink_freq.frq"
Var_Path_log_out = Var_Path_Output+"log_out"
Var_Path_Result_out = Var_Path_Output+"Selected_SNP"
Var_Path_SNP_Name = Var_Path_Output+"snp_name.txt"
Var_Path_Update_Chr=Var_Path_Output+"Update_Chr"
Var_Path_Update_Id=Var_Path_Output+"Update_Id"


##### Ouverture en écriture du fichier log_out

Log_out_File = open(Var_Path_log_out, "w")

Log_out_File.write("#### Analysis parameters:\n")
Log_out_File.write("\n")
Log_out_File.write("Map file:        " + Var_Path_Map+"\n")
Log_out_File.write("Genotype file:   "+ Var_Path_Genotype+"\n")
Log_out_File.write("LD Threshold:    " + str(Var_Threshold)+"\n")
Log_out_File.write("Chromosome:      " + str(Var_Chr)+"\n")
Log_out_File.write("SNP window:      " + str(Var_Seuil_Nb_SNP)+" SNP.\n")
Log_out_File.write("\n")
Log_out_File.write("\n")
Log_out_File.write("##########################################################\n")
Log_out_File.write("\n")
Log_out_File.write("\n")
Log_out_File.close()

##### Ouverture en écriture du fichier resultat
Output_Result_File = open(Var_Path_Result_out, "w")
Output_Result_File.write("SNP Name \t Chromosome \t Position \t MAF \t Number of clustered SNP \t Cluster minimum R2 \t Cluster average R2 \t Cluster standard deviation R2 \t Selected SNP minimum R2 \t Selected SNP maximum R2 \t Selected SNP average R2 \t Selected SNP standard deviation R2 \t Cluster minimum distance between SNP pairs \t  Cluster maximum distance between SNP pairs \t  Cluster average distance between SNP pairs \t  Cluster standard deviation distance between SNP pairs \t Selected SNP minimum distance between SNP pairs \t Selected SNP maximum distance between SNP pairs \t Selected SNP average distance between SNP pairs \t Selected SNP standard deviation distance between SNP pairs\n")
Output_Result_File.close()

##### génère les bfile plink
## génère un fichier pour modifier les position chromosome dans le bed file. Ainsi plus de problème de .hh
Input_File = open(Var_Path_Map, "r")
Output_File=open (Var_Path_Update_Chr, "w")
for ligne in Input_File:
	ligne = ligne.split()
	Output_File.write (ligne [1]+'\t0\n')
Output_File.close()
Input_File.close()

## génère un fichier pour modifier les individus comme fondateur dans le bed file.
# Si un ou plusieur SNP ne sont pas génotypé chez des fondateur (pere 0 et mere 0) alors ce(s) SNP n'est pas pris en compte dans les analyses de PLINK
# Cela pose problème quand le pedigree complet est renseigné dans le fichier ped
# On  modifie donc le fichier pour placer l'ensemble des individus du fichier en fondateur.
Input_File = open(Var_Path_Genotype, "r")
Output_File=open (Var_Path_Update_Id, "w")
for ligne in Input_File:
	ligne = ligne.split()
	Output_File.write (ligne [0]+'\t'+ligne[1]+'\t0\t0\n')
Output_File.close()
Input_File.close()

## génère le bed file
Var_Sortie_Plink_bfile = Var_Path_Output+"bfile"

subprocess.call(["plink-1-09", "--ped", Var_Path_Genotype, "--map", Var_Path_Map, "--update-parents",Var_Path_Update_Id, "--update-chr", Var_Path_Update_Chr, "--chr-set", "95", "no-xy", "no-mt", "--allow-extra-chr", "0", "--make-bed", "--out", Var_Sortie_Plink_bfile ])

##### Boucle sur l'ensemble des chromosomes à analyser

for Var_Chrom_Name in List_Chromosome_to_Analyze:

	##### Determine le nombre de passage necessaire pour analyser le Var_Chrom_Name
	List_Tuple_Position_SNPName=Get_List_Tuple_Position_SNPName(Var_Chrom_Name, Var_Path_Map)
	Var_Nb_Analyse_Chr=int(len(List_Tuple_Position_SNPName)/Var_Seuil_Nb_SNP+1)
	Var_Nb_SNP_to_Analyze=int(len(List_Tuple_Position_SNPName)/Var_Nb_Analyse_Chr)+1

	##### Ecriture dans le fichier log out
	Log_out_File = open(Var_Path_log_out, "a")
	Log_out_File.write("# Chromosome "+ str(Var_Chrom_Name)+"\n")
	if Var_Nb_Analyse_Chr>1:
		Log_out_File.write("The chromosome was processed in "+ str(Var_Nb_Analyse_Chr) +" windows.\n")
	else:
		Log_out_File.write("The chromosome was processed in a single operation.\n")
	Log_out_File.close()

	##### Boucle sur les différentes fenêtre d'analyse pour le chromosome
	for window in range (Var_Nb_Analyse_Chr):
		if Var_Nb_Analyse_Chr>1:
			Log_out_File = open(Var_Path_log_out, "a")
			Log_out_File.write("Window " + str (window+1)+".\n")
			Log_out_File.close()

		##### Creation des fichiers R2_matrix et Fréquence en utilisant PLINK
		Extract_SNP_List_for_PLINK(List_Tuple_Position_SNPName, Var_Nb_SNP_to_Analyze, Var_Path_Output)
		List_Tuple_Position_SNPName=List_Tuple_Position_SNPName[Var_Nb_SNP_to_Analyze:]
		Var_heure = Print_time(time.localtime())
		Log_out_File = open(Var_Path_log_out, "a")
		Log_out_File.write("Call PLINK freq "+Var_heure+"\n")
		Log_out_File.close()
		Var_Sortie_Plink_Freq = Var_Path_Output+"sortie_plink_freq"
		subprocess.call(["plink-1-09", "--bfile", Var_Sortie_Plink_bfile, "--freq", "--extract", Var_Path_SNP_Name, "--out", Var_Sortie_Plink_Freq])
		del Var_Sortie_Plink_Freq
		Var_heure = Print_time(time.localtime())
		Log_out_File = open(Var_Path_log_out, "a")
		Log_out_File.write("Call PLINK LD (r2)  "+Var_heure+"\n")
		Log_out_File.close()
		Var_Sortie_Plink_R2 = Var_Path_Output+"sortie_plink_r2"
		subprocess.call(["plink-1-09", "--bfile", Var_Sortie_Plink_bfile, "--r2", "square", "spaces", "--extract", Var_Path_SNP_Name, "--out", Var_Sortie_Plink_R2])
		del Var_Sortie_Plink_R2
		Var_heure = Print_time(time.localtime())
		Log_out_File = open(Var_Path_log_out, "a")
		Log_out_File.write("Get MAF and DL from PLINK files "+Var_heure+"\n")
		Log_out_File.close()



		##### Lecture et importation des données R2, Frequence et position
		List_data_R2, Var_Nb_SNP = Import_R2_Matrix(Var_Path_R2_File)
		List_SNP_Name, List_MAF = Import_MAF(Var_Path_Freq_File)
		List_Position = Extract_SNP_Position(Var_Chrom_Name,Var_Path_Map)



		##### Netoyage des donnes en fonction du seuil de R2
		List_Index_SNP_to_Delete = Get_Index_of_SNP_to_Delete(List_data_R2, Var_Nb_SNP, Var_Threshold)
		List_data_R2, List_SNP_Name, List_MAF, List_Position, List_Name_SNP_Deleted = Cleaning_Data(List_data_R2, List_SNP_Name, List_MAF, List_Position, List_Index_SNP_to_Delete)
		Var_Nb_SNP=len(List_data_R2)
		############### si la liste List_SNP_Name qui contient les SNP qui restent dans la matrice est > 2 alors continuer l'analyse. Sinon passer au chr suivant. Deux SNP est le nombre minimum de relation pour une matrice r2.'
		if (Var_Nb_SNP>2):
			Log_out_File = open(Var_Path_log_out, "a")
			Log_out_File.write("Removed SNP "+str(List_Name_SNP_Deleted)+"\n")



			##### Hierarchical clustering and dendogram
			Var_heure = Print_time(time.localtime())
			Log_out_File.write("Dendrogram computation "+Var_heure+"\n")
			Log_out_File.close()

			List_flat_distance_array = List_data_R2_to_Flat_Distance_Array(List_data_R2)
			del List_data_R2 #Nettoyage mémoire
			del Var_Nb_SNP #Nettoyage mémoire
			del List_SNP_Name #Nettoyage mémoire
			del List_MAF #Nettoyage mémoire
			del List_Position #Nettoyage mémoire
			Z = linkage(List_flat_distance_array, method="average")
			del List_flat_distance_array #Nettoyage mémoire



			##### Lecture et importation des données R2, Frequence et position
			List_data_R2, Var_Nb_SNP = Import_R2_Matrix(Var_Path_R2_File)
			List_SNP_Name, List_MAF = Import_MAF(Var_Path_Freq_File)
			List_Position = Extract_SNP_Position(Var_Chrom_Name,Var_Path_Map)



			##### Netoyage des donnes en fonction du seuil de R2
			List_Index_SNP_to_Delete = Get_Index_of_SNP_to_Delete(List_data_R2, Var_Nb_SNP, Var_Threshold)
			List_data_R2, List_SNP_Name, List_MAF, List_Position, List_Name_SNP_Deleted = Cleaning_Data(List_data_R2, List_SNP_Name, List_MAF, List_Position, List_Index_SNP_to_Delete)
			Var_Nb_SNP=len(List_data_R2)



			######## Parcours du dendrogramme et selection des clusters ainsi que du SNP représentatif
			Log_out_File = open(Var_Path_log_out, "a")
			Var_heure = Print_time(time.localtime())
			Log_out_File.write("Cluster identification and SNP selection "+Var_heure+"\n")
			Log_out_File.close()
			Cluster_Selection(Z, List_data_R2, Var_Threshold)
		else:
			Log_out_File = open(Var_Path_log_out, "a")
			Log_out_File.write("Removed SNP "+str(List_Name_SNP_Deleted)+"\n")
			Log_out_File.write("There is no more SNP to select.\n")
			Log_out_File.close()

	######## Ecriture fin d'analyse

	Var_heure = Print_time(time.localtime())
	Log_out_File = open(Var_Path_log_out, "a")
	Log_out_File.write("Chromosome "+str(Var_Chrom_Name)+" analysis completed at "+Var_heure+"\n\n\n")
	Log_out_File.close()




##### Suppression des fichiers temporaires
Var_2_delete = Var_Path_Output+"bfile.log"
#Controle de l'existence du fichier et le supprime si il existe
if os.path.exists(Var_2_delete):
	os.remove(Var_2_delete)

Var_2_delete = Var_Path_Output+"bfile.bed"
if os.path.exists(Var_2_delete):
	os.remove(Var_2_delete)

Var_2_delete = Var_Path_Output+"bfile.hh"
if os.path.exists(Var_2_delete):
	os.remove(Var_2_delete)

Var_2_delete = Var_Path_Output+"bfile.fam"
if os.path.exists(Var_2_delete):
	os.remove(Var_2_delete)

Var_2_delete = Var_Path_Output+"bfile.bim"
if os.path.exists(Var_2_delete):
	os.remove(Var_2_delete)

Var_2_delete = Var_Path_Output+"bfile.nosex"
if os.path.exists(Var_2_delete):
	os.remove(Var_2_delete)

Var_2_delete = Var_Path_Output+"sortie_plink_freq.hh"
if os.path.exists(Var_2_delete):
	os.remove(Var_2_delete)

Var_2_delete = Var_Path_Output+"sortie_plink_freq.frq"
if os.path.exists(Var_2_delete):
	os.remove(Var_2_delete)

Var_2_delete = Var_Path_Output+"sortie_plink_freq.log"
if os.path.exists(Var_2_delete):
	os.remove(Var_2_delete)

Var_2_delete = Var_Path_Output+"sortie_plink_freq.nosex"
if os.path.exists(Var_2_delete):
	os.remove(Var_2_delete)

Var_2_delete = Var_Path_Output+"sortie_plink_r2.hh"
if os.path.exists(Var_2_delete):
	os.remove(Var_2_delete)

Var_2_delete = Var_Path_Output+"sortie_plink_r2.ld"
if os.path.exists(Var_2_delete):
	os.remove(Var_2_delete)

Var_2_delete = Var_Path_Output+"sortie_plink_r2.log"
if os.path.exists(Var_2_delete):
	os.remove(Var_2_delete)

Var_2_delete = Var_Path_Output+"sortie_plink_r2.nosex"
if os.path.exists(Var_2_delete):
	os.remove(Var_2_delete)

Var_2_delete = Var_Path_Output+"snp_name.txt"
if os.path.exists(Var_2_delete):
	os.remove(Var_2_delete)

Var_2_delete = Var_Path_Output+"map.txt"
if os.path.exists(Var_2_delete):
	os.remove(Var_2_delete)

Var_2_delete = Var_Path_Update_Chr
if os.path.exists(Var_2_delete):
	os.remove(Var_2_delete)

Var_2_delete = Var_Path_Update_Id
if os.path.exists(Var_2_delete):
	os.remove(Var_2_delete)

